@@ -49,7 +49,7 @@ expose_stan_functions <- function(stanmodel) {
     
   # get rid of templating and just use double because that is about all R can pass
   lines <- gsub("typename boost::math::tools::promote_args.*type ", "double ", lines)
-  lines <- gsub("^((std::vector<)+)typename boost::math::tools::promote_args.*(>::type)+", "\\1double", lines)
+  lines <- gsub("((std::vector<)+)typename boost::math::tools::promote_args.*(>::type)+", "\\1double", lines)
   lines <- gsub("vector<Eigen::Matrix<.*Eigen::Dynamic,1> >", "vector<vector_d>", lines)
   lines <- gsub("Eigen::Matrix<.*Eigen::Dynamic,1>", "vector_d", lines)
   lines <- gsub("vector<Eigen::Matrix<.*1,Eigen::Dynamic> >", "vector<row_vector_d>", lines)
@@ -80,16 +80,24 @@ expose_stan_functions <- function(stanmodel) {
     if(grepl("_rng", lines[i], fixed = TRUE)) {
       lines[i] <- gsub("RNG\\&.*\\{$", "const int& seed = 0) {", lines[i])
       lines <- append(lines, c("static boost::ecuyer1988 base_rng__(seed);", 
-                               "std::ostream* pstream__ = &Rcpp::Rcout;"), i)
+                               "std::ostream* pstream__ = &Rcpp::Rcout;", 
+                               "(void) pstream__;"), i)
     }
     else if(grepl("_lp", lines[i], fixed = TRUE)) {
       lines[i] <- gsub(", T_lp_accum__\\&.*\\{$", ") {", lines[i])
       lines <- append(lines, c("stan::math::accumulator<double> lp_accum__;",
-                               "std::ostream* pstream__ = &Rcpp::Rcout;"), i)
+                               "std::ostream* pstream__ = &Rcpp::Rcout;", 
+                               "(void) pstream__;"), i)
+    }
+    else if(grepl("(std::ostream* pstream__) {", lines[i], fixed = TRUE)) {
+      lines[i] <- gsub("(std::ostream* pstream__) {", "() {", lines[i], fixed = TRUE)
+      lines <- append(lines, c("std::ostream* pstream__ = &Rcpp::Rcout;", 
+                               "(void) pstream__;"), i)
     }
     else {
       lines[i] <- gsub(", std::ostream* pstream__) {", ") {", lines[i], fixed = TRUE)
-      lines <- append(lines, "std::ostream* pstream__ = &Rcpp::Rcout;", i)
+      lines <- append(lines, c("std::ostream* pstream__ = &Rcpp::Rcout;", 
+                               "(void) pstream__;"), i)
     }
     lines <- append(lines, usings, i) # make the usings:: local to the function
   }
@@ -98,13 +106,14 @@ expose_stan_functions <- function(stanmodel) {
   lines <- grep("^using stan::io::", lines, value = TRUE, invert = TRUE)
   lines <- grep("^using stan::model::prob_grad", lines, value = TRUE, invert = TRUE)
   
-  # get rid of inline declarations
-  lines <- grep("^inline", lines, value = TRUE, invert = TRUE)
-  
+  # convert inline declarations to Rcpp export declarations
+  lines <- gsub("^inline$", "// \\[\\[Rcpp::export\\]\\]", lines)
+
   # declare attributes for Rcpp for non-functor user-defined Stan functions
   templates <- grep("^template .*$", lines)
-  for(i in rev(templates)) if(!grepl("functor__", lines[i - 1L])) {
-    lines <- append(lines, "// [[Rcpp::export]]", i - 1L)
+  for(i in rev(templates)) {
+    if(!grepl("functor__", lines[i - 1L]) && lines[i + 1L] != "// [[Rcpp::export]]")
+      lines <- append(lines, "// [[Rcpp::export]]", i - 1L)
   }
   
   # do not export function declarations created by the user
@@ -116,31 +125,51 @@ expose_stan_functions <- function(stanmodel) {
     while(lines[j] != "// [[Rcpp::export]]") j <- j - 1L
     lines <- lines[-j]
   }
+
+  # special case for integrate_ode()
+  ODE_lines <- grep("integrate_ode(", lines, fixed = TRUE)
+  ODE_statements <- grep("integrate_ode(", lines, fixed = TRUE, value = TRUE)
   
-  # remove more pstream__ arguments
-  lines <- gsub(", std::ostream* pstream__) const {", ") const {", 
-                lines, fixed = TRUE)
+  # handle more pstream__ arguments
   lines <- gsub(", pstream__)", ")", lines, fixed = TRUE)
+  lines <- gsub("(pstream__)", "()", lines, fixed = TRUE)
+  lines <- gsub(", std::ostream* pstream__) const {",
+                ", std::ostream* pstream__ = &Rcpp::Rcout) const {",
+                lines, fixed = TRUE)
+  lines <- gsub("(std::ostream* pstream__)", 
+                "(std::ostream* pstream__ = &Rcpp::Rcout)", 
+                lines, fixed = TRUE)
+  
+  # put back pstream__ arguments in the case of ODEs
+  if (length(ODE_lines) > 0) lines[ODE_lines] <- ODE_statements
+
+  lines <- gsub("typename boost::math::tools::promote_args.*(>::type)+", "double", lines)
   
   # remove more base_rng__ arguments
   lines <- gsub(", RNG& base_rng__", "", 
                 lines, fixed = TRUE)
+  lines <- gsub("(RNG& base_rng__,", "(",
+                lines, fixed = TRUE)
   lines <- gsub("(RNG& base_rng__", "(",
                 lines, fixed = TRUE)
   lines <- gsub("([[:space:]]+return .*_rng.*), base_rng__\\);",
                 "\\1);", lines)
   lines <- gsub("([[:space:]]+return .*_rng)\\(base_rng__\\);",
                 "\\1();", lines)
+  
+  # remove line numbering things
+  lines <- grep("current_statement_begin__", lines, 
+                fixed = TRUE, value = TRUE, invert = TRUE)
                   
   # replace more templating with doubles
   lines <- gsub("const T[0-9]+__&", "const double&", lines)
   lines <- gsub("T_lp__& lp__", "double lp__ = 0.0", lines)
   lines <- gsub("^typename.*$", "double", lines)
-  lines <- grep("^template", lines, invert = TRUE, value = TRUE)
+  lines <- grep("^[[:space:]]*template", lines, invert = TRUE, value = TRUE)
   lines <- gsub("<T[0-9]+__>", "<double>", lines)
   
   # deal with accumulators
-  lines <- gsub(", T_lp_accum__& lp_accum__)", ")", lines, fixed = TRUE)
+  lines <- gsub(", T_lp_accum__& lp_accum__", "", lines, fixed = TRUE)
   lines <- gsub(", lp_accum__", "", lines, fixed = TRUE)
   lines <- gsub("get_lp(lp__)", "get_lp(lp__, lp_accum__)", lines, fixed = TRUE)
   
